/****************************************************************************

 Module
  docalls.c
  
 Description
  AES routine identification and distribution in oAESis.
  
 Author(s)
 	cg (Christer Gustavsson <d2cg@dtek.chalmers.se>)

 Revision history
 
  960103 cg
   Added standard header.
   Fsel_exinput() added.
 
 Copyright notice
  The copyright to the program code herein belongs to the authors. It may
  be freely duplicated and distributed without fee, but not charged for.
 
 ****************************************************************************/

/****************************************************************************
 * Used interfaces                                                          *
 ****************************************************************************/

#include	<stdio.h>

#include "appl.h"
#include "debug.h"
#include "evnt.h"
#include "form.h"
#include "fsel.h"
#include "global.h"
#include "graf.h"
#include "menu.h"
#include "objc.h"
#include "rdwrs.h"
#include "rsrc.h"
#include "scrp.h"
#include "shel.h"
#include "types.h"
#include "wind.h"

/****************************************************************************
 * Public functions                                                         *
 ****************************************************************************/

void h_aes_call(AES_PB *apb) {
/*	char s[200];

	
	sprintf(s,"aes call: %x",apb->control[0]);
	DEBUG(s);
*/
/*
#define DCDEBUG
*/
#ifdef DCDEBUG
	if(apb->global[2] != 0) {
		fprintf(stderr,"Aes call: %d\r\n",apb->control[0]);
	};
#endif
	
	switch(apb->control[0]) {
		case	0x000a:	/*appl_init*/
			Appl_init(apb);
			break;
		case	0x000b:	/*appl_read*/
			Appl_read(apb);
			break;
		case	0x000c:	/*appl_write*/
			Appl_write(apb);
			break;
		case 0x000d: /*appl_find*/
			Appl_find(apb);
			break;
		case 0x0012: /*appl_search*/
			Appl_search(apb);
			break;
		case	0x0013:	/*appl_exit*/
			Appl_exit(apb);
			break;

		case	0x0014:	/*evnt_keybd*/
			Evnt_keybd(apb);
			break;
		case	0x0015:	/*evnt_button*/
			Evnt_button(apb);
			break;
		case	0x0017:	/*evnt_mesag*/
			Evnt_mesag(apb);
			break;
		case	0x0018:	/*evnt_timer*/
			Evnt_timer(apb);
			break;
		case	0x0019:	/*evnt_multi*/
			Evnt_multi(apb);
			break;
		case	0x001a:	/*evnt_dclick*/
			Evnt_dclick(apb);
			break;

		case	0x001e:	/*menu_bar*/
			Menu_bar(apb);
			break;
		case	0x001f:	/*menu_icheck*/
			Menu_icheck(apb);
			break;
		case	0x0020:	/*menu_ienable*/
			Menu_ienable(apb);
			break;
		case	0x0021:	/*menu_tnormal*/
			Menu_tnormal(apb);
			break;
		case	0x0022:	/*menu_text*/
			Menu_text(apb);
			break;
		case	0x0023:	/*menu_register*/
			Menu_register(apb);
			break;

		case	0x0028:	/*objc_add*/
			Objc_add(apb);
			break;
		case	0x0029:	/*objc_delete*/
			printf("objc_delete");
			break;
		case	0x002a:	/*objc_draw*/
			Objc_draw(apb);
			break;
		case	0x002b:	/*objc_find*/
			Objc_find(apb);
			break;
		case	0x002c:	/*objc_offset*/
			Objc_offset(apb);
			break;
		case 0x002e: /*objc_edit*/
			Objc_edit(apb);
			break;
		case 0x002f: /*objc_change*/
			Objc_change(apb);
			break;
			
		case	0x0032:	/*form_do*/
			Form_do(apb);
			break;
		case	0x0033:	/*form_dial*/
			Form_dial(apb);
			break;
		case	0x0034:	/*form_alert*/
			Form_alert(apb);
			break;
		case	0x0035:	/*form_error*/
			Form_error(apb);
			break;
		case	0x0036:	/*form_center*/
			Form_center(apb);
			break;
		case	0x0037:	/*form_keybd*/
			Form_keybd(apb);
			break;
		case	0x0038:	/*form_button*/
			Form_button(apb);
			break;

		case 0x0047: /*graf_dragbox*/
			Graf_dragbox(apb);
			break;
		case 0x0049: /*graf_growbox*/
			Graf_growbox(apb);
			break;
		case 0x004a: /*graf_shrinkbox*/
			Graf_shrinkbox(apb);
			break;
		case	0x004b: /*graf_watchbox*/
			Graf_watchbox(apb);
			break;
		case 0x004c: /*graf_slidebox*/
			Graf_slidebox(apb);
			break;
		case	0x004d:	/*graf_handle*/
			Graf_handle(apb);
			break;
		case	0x004e:	/*graf_mouse*/
			Graf_mouse(apb);
			break;
		case	0x004f:	/*graf_mkstate*/
			Graf_mkstate(apb);
			break;

		case 0x0050: /*scrp_read*/
			Scrp_read(apb);
			break;
		case 0x0051: /*scrp_write*/
			Scrp_write(apb);
			break;
			
		case 0x005a: /*fsel_input*/
			Fsel_input(apb);
			break;
		case 0x005b: /*fsel_exinput*/
			Fsel_exinput(apb);
			break;

		case	0x0064:	/*wind_create*/
			Wind_create(apb);
			break;
		case	0x0065:	/*wind_open*/
			Wind_open(apb);
			break;
		case	0x0066:	/*wind_close*/
			Wind_close(apb);
			break;
		case	0x0067:	/*wind_delete*/
			Wind_delete(apb);
			break;
		case	0x0068:	/*wind_get*/
			Wind_get(apb);
			break;
		case	0x0069:	/*wind_set*/
			Wind_set(apb);
			break;
		case	0x006a:	/*wind_find*/
			Wind_find(apb);
			break;
		case	0x006b:	/*wind_update*/
			Wind_update(apb);
			break;
		case	0x006c:	/*wind_calc*/
			Wind_calc(apb);
			break;

		case	0x006e:	/*rsrc_load*/
			Rsrc_load(apb);
			break;
		case	0x006f:	/*rsrc_free*/
			Rsrc_free(apb);
			break;
		case	0x0070:	/*rsrc_gaddr*/
			Rsrc_gaddr(apb);
			break;
		case	0x0071:	/*rsrc_saddr*/
			Rsrc_saddr(apb);
			break;
		case	0x0072:	/*rsrc_obfix*/
			Rsrc_obfix(apb);
			break;
		case	0x0073:	/*rsrc_rcfix*/
			Rsrc_rcfix(apb);
			break;
			
		case 0x0078: /*shel_read*/
			Shel_read(apb);
			break;
 		case 0x0079: /*shel_write*/
 			Shel_write(apb);		
 			break;
		case 0x007c: /*shel_find*/
			Shel_find(apb);
			break;
	
		case 0x0082: /*appl_getinfo*/
			Appl_getinfo(apb);
			break;
			
		default:
			DEBUG("Not implemented call!");
			DEBUG(s);
			fprintf(stderr,"%s: Line %d:\r\n"
							"Unimplemented AES call %d\r\n",
							__FILE__,__LINE__,apb->control[0]);
							
			sleep(1);
	};
	
#ifdef DCDEBUG
	if(apb->global[2] != 0) {
		fprintf(stderr,"End aes call: %d\r\n",apb->control[0]);
	};
#endif
}

