/****************************************************************************

 Module
  graf.c
  
 Description
  Dynamic graphical routines in oAESis.
  
 Author(s)
 	cg (Christer Gustavsson <d2cg@dtek.chalmers.se>)

 Revision history
 
  960103 cg
   Added standard header.
 
 Copyright notice
  The copyright to the program code herein belongs to the authors. It may
  be freely duplicated and distributed without fee, but not charged for.
 
 ****************************************************************************/

/****************************************************************************
 * Used interfaces                                                          *
 ****************************************************************************/

#include <aesbind.h>
#include <mintbind.h>
#include <osbind.h>
#include <stdio.h>

#include "appl.h"
#include "evnt.h"
#include "evnthndl.h"
#include "misc.h"
#include "objc.h"
#include "resource.h"
#include "types.h"
#include "vdi.h"
#include "wind.h"

/****************************************************************************
 * Module global variables                                                  *
 ****************************************************************************/

static MFORM m_arrow,m_text_crsr,m_busy_bee,m_flat_hand;

/****************************************************************************
 * Local functions (use static!)                                            *
 ****************************************************************************/

void icon2mform(MFORM *mf,OBJECT *icon) {
	WORD	i;
	
	/*convert resource icons to MFORM's*/
	
	mf->mf_xhot = 0;
	mf->mf_yhot = 0;
	mf->mf_nplanes = 1;
	mf->mf_fg = 1;
	mf->mf_bg = 0;
	
	for(i = 0; i < 16; i++) {
		mf->mf_mask[i] = ((ICONBLK *)icon->ob_spec)->ib_pmask[i];
		mf->mf_data[i] = ((ICONBLK *)icon->ob_spec)->ib_pdata[i];
	};
}

/****************************************************************************
 * Public functions                                                         *
 ****************************************************************************/
 
void	init_graf(void) {
	icon2mform(&m_arrow,&mouseformstad[MARROW]);
	icon2mform(&m_text_crsr,&mouseformstad[MTEXT_CRSR]);
	icon2mform(&m_busy_bee,&mouseformstad[MBUSY_BEE]);
	icon2mform(&m_flat_hand,&mouseformstad[MFLAT_HAND]);
}


/****************************************************************************
 * Graf_do_dragbox                                                          *
 *  Implementation of graf_dragbox().                                       *
 ****************************************************************************/
WORD              /* 1 if ok or 0.                                          */
Graf_do_dragbox(  /*                                                        */
WORD   apid,      /* Application id.                                        */
WORD   w,         /* Width of box.                                          */
WORD   h,         /* Height of box.                                         */
WORD   sx,        /* Starting x.                                            */
WORD   sy,        /* Starting y.                                            */
RECT   *bound,    /* Bounding rectangle.                                    */
WORD   *endx,     /* Ending x.                                              */
WORD   *endy)     /* Ending y.                                              */
/****************************************************************************/
{
	WORD    eventpipe = Appl_get_eventpipe(apid);
	WORD    vid = Appl_get_vid(apid);
	EVNTREC er;
	WORD    relx = sx - globals.mouse_x;
	WORD    rely = sy - globals.mouse_y;
	WORD    xyarray[10];
	WORD    lastx = sx,lasty = sy;
	WORD    newx,newy;

	if(!(globals.mouse_button & LEFT_BUTTON)) {	
		*endx = sx;
		*endy = sy;
	
		return 1;
	};
	
	Evhd_gain_mctrl(apid,MCTRL_COMPLETE);

	Vdi_vswr_mode(vid,MD_XOR);
  Vdi_vsl_type(vid,DOTTED);

  xyarray[0] = lastx;
  xyarray[1] = lasty;
  xyarray[2] = lastx + w - 1;
  xyarray[3] = xyarray[1];
  xyarray[4] = xyarray[2];
  xyarray[5] = lasty + h - 1;
  xyarray[6] = xyarray[0];
  xyarray[7] = xyarray[5];
  xyarray[8] = xyarray[0];
  xyarray[9] = xyarray[1];
	 
	Vdi_v_hide_c(vid);
  Vdi_v_pline(vid,5,xyarray);
  Vdi_v_show_c(vid,1);
	
	while(Fread(eventpipe,sizeof(EVNTREC),&er)) {
		if(er.ap_event == APPEVNT_MOUSE) {
			while(Finstat(eventpipe) >= sizeof(EVNTREC)) {
				Fread(eventpipe,sizeof(EVNTREC),&er);
				
				if(er.ap_event != APPEVNT_MOUSE) {
					break;
				};
				
				newx = ((WORD *)&er.ap_value)[1] + relx;
    		newy = ((WORD *)&er.ap_value)[0] + rely;
			};

			if(newx < bound->x) {
				newx = bound->x;
			}
			else if(newx > (bound->x + bound->width - w)) {
				newx = bound->x + bound->width - w;
			};

			if(newy < bound->y) {
				newy = bound->y;
			}
			else if(newy > (bound->y + bound->height - h)) {
				newy = bound->y + bound->height - h;
			};

			if((lastx != newx) || (lasty != newy)) {
	      Vdi_v_hide_c(vid);
	      Vdi_v_pline(vid,5,xyarray);

 		    xyarray[0] = newx;
	      xyarray[1] = newy;
	      xyarray[2] = newx + w - 1;
	      xyarray[3] = xyarray[1];
	      xyarray[4] = xyarray[2];
	      xyarray[5] = newy + h - 1;
	      xyarray[6] = xyarray[0];
	      xyarray[7] = xyarray[5];
	      xyarray[8] = xyarray[0];
	      xyarray[9] = xyarray[1];
	 
	      Vdi_v_pline(vid,5,xyarray);
	      Vdi_v_show_c(vid,1);
	      
	      lastx = newx;
	      lasty = newy;
	    };
		};
		
		if(er.ap_event == APPEVNT_BUTTON) {
			if(!(er.ap_value & LEFT_BUTTON)) {
				break;
			};
		};
	};

  Vdi_v_hide_c(vid);
  Vdi_v_pline(vid,5,xyarray);
	Vdi_v_show_c(vid,1);
	
	Evhd_release_mctrl(apid);

	*endx = lastx;
	*endy = lasty;
	
	return 1;
}

/****************************************************************************
 * Graf_dragbox                                                             *
 *  0x0047 graf_dragbox().                                                  *
 ****************************************************************************/
void              /*                                                        */
Graf_dragbox(     /*                                                        */
AES_PB *apb)      /* AES parameter block.                                   */
/****************************************************************************/
{
	apb->int_out[0] = Graf_do_dragbox(apb->global[2],apb->int_in[0],
										apb->int_in[1],apb->int_in[2],apb->int_in[3],
										(RECT *)&apb->int_in[4],&apb->int_out[1],
										&apb->int_out[2]);
}

/****************************************************************************
 * Graf_growbox                                                             *
 *  0x0049 graf_growbox().                                                  *
 ****************************************************************************/
void              /*                                                        */
Graf_growbox(     /*                                                        */
AES_PB *apb)      /* AES parameter block.                                   */
/****************************************************************************/
{
	apb->int_out[0] = 1;
}

/****************************************************************************
 * Graf_shrinkbox                                                           *
 *  0x004a graf_shrinkbox().                                                *
 ****************************************************************************/
void              /*                                                        */
Graf_shrinkbox(   /*                                                        */
AES_PB *apb)      /* AES parameter block.                                   */
/****************************************************************************/
{
	apb->int_out[0] = 1;
}

/****************************************************************************
 * Graf_do_watchbox                                                         *
 *  Implementation of graf_watchbox().                                      *
 ****************************************************************************/
WORD              /* 1 if inside object when button was released or 0.      */
Graf_do_watchbox( /*                                                        */
WORD   apid,      /* Application id.                                        */
OBJECT *tree,     /* Resource tree.                                         */
WORD   obj,       /* Object to watch.                                       */
WORD   instate,   /* State when inside object.                              */
WORD   outstate)  /* State when outside object.                             */
/****************************************************************************/
{
  EVENTIN ei = {
    MU_BUTTON | MU_M1,
    0,
    LEFT_BUTTON,
    0,
    0,
    {0,0,0,0},
    0,
    {0,0,0,0},
    0,
    0
    };
							
	EVENTOUT	eo;
	COMMSG    buffer;
	
	WORD vid = Appl_get_vid(apid);

	Objc_do_offset(tree,obj,(WORD *)&ei.m1r);
	ei.m1r.width = tree[obj].ob_width;
	ei.m1r.height = tree[obj].ob_height;
	
	if(Misc_inside(&ei.m1r,globals.mouse_x,globals.mouse_y)) {
		ei.m1flag = MO_LEAVE;
		
		Objc_do_change(vid,tree,obj,&ei.m1r,instate,REDRAW);
	}
	else {
		ei.m1flag = MO_ENTER;

		Objc_do_change(vid,tree,obj,&ei.m1r,outstate,REDRAW);
	};
	
	Evhd_gain_mctrl(apid,MCTRL_BUTTON);
	
	while(1) {
		Evnt_do_multi(apid,&ei,&buffer,&eo,0);
		
		if(eo.events & MU_BUTTON) {
			break;
		};
		
		if(ei.m1flag == MO_LEAVE) {
			ei.m1flag = MO_ENTER;
	
			Objc_do_change(vid,tree,obj,&ei.m1r,outstate,REDRAW);
		}
		else {
			ei.m1flag = MO_LEAVE;

			Objc_do_change(vid,tree,obj,&ei.m1r,instate,REDRAW);
		};
	};
	
	Evhd_release_mctrl(apid);
					
	if(ei.m1flag == MO_LEAVE) {
		return 1;
	};
	
	return 0;
}

/****************************************************************************
 * Graf_watchbox                                                            *
 *  0x004b graf_watchbox().                                                 *
 ****************************************************************************/
void              /*                                                        */
Graf_watchbox(    /*                                                        */
AES_PB *apb)      /* AES parameter block.                                   */
/****************************************************************************/
{
	apb->int_out[0] = Graf_do_watchbox(apb->global[2],(OBJECT *)apb->addr_in[0],
										apb->int_in[1],apb->int_in[2],apb->int_in[3]);
}

/****************************************************************************
 * Graf_do_slidebox                                                         *
 *  Implementation of graf_slidebox().                                      *
 ****************************************************************************/
WORD              /* Relative offset.                                       */
Graf_do_slidebox( /*                                                        */
WORD   apid,      /* Application id.                                        */
OBJECT *tree,     /* Resource tree.                                         */
WORD   parent,    /* Parent object.                                         */
WORD   obj,       /* Slider object.                                         */
WORD   orient)    /* Orientation. 0 => horizontal, 1 => vertical.           */
/****************************************************************************/
{
	RECT bound;
	WORD x,y;
	WORD sxy[2];
	
	if(!tree) {
		return 0;
	};
	
	Objc_do_offset(tree,obj,sxy);
	Objc_do_offset(tree,parent,(WORD *)&bound);
	bound.width = tree[parent].ob_width;
	bound.height = tree[parent].ob_height;
	
	Graf_do_dragbox(apid,tree[obj].ob_width,tree[obj].ob_height,
									sxy[0],sxy[1],&bound,&x,&y);
	
	if(orient == 0) {
		if(tree[obj].ob_width != tree[parent].ob_width) {
			return (WORD)((((LONG)x - (LONG)bound.x) * 1000L) /
										((LONG)(tree[parent].ob_width - tree[obj].ob_width)));
		};
	}
	else {
		if(tree[obj].ob_height != tree[parent].ob_height) {
			return (WORD)((((LONG)y - (LONG)bound.y) * 1000L) /
										((LONG)(tree[parent].ob_height - tree[obj].ob_height)));
		};
	};
									
	return 0;
}

/****************************************************************************
 * Graf_slidebox                                                            *
 *  0x004c graf_slidebox().                                                 *
 ****************************************************************************/
void              /*                                                        */
Graf_slidebox(    /*                                                        */
AES_PB *apb)      /* AES parameter block.                                   */
/****************************************************************************/
{
	apb->int_out[0] = Graf_do_slidebox(apb->global[2],(OBJECT *)apb->addr_in[0],
										apb->int_in[0],apb->int_in[1],apb->int_in[2]);
}

/*graf_handle 0x004d*/

WORD	do_graf_handle(WORD *cwidth,WORD *cheight,WORD *width
							,WORD *height) {
	*cwidth = globals.cwidth;
	*cheight = globals.cheight;
	*width = globals.bwidth;
	*height = globals.bheight;
	
	return globals.vid;
}

void	Graf_handle(AES_PB *apb) {
	apb->int_out[0] = do_graf_handle(&apb->int_out[1]
								,&apb->int_out[2],&apb->int_out[3]
								,&apb->int_out[4]);	
}

/*graf_mouse 0x004e*/
void	Graf_mouse(AES_PB *apb) {
	WORD	vid = Appl_get_vid(apb->global[2]);
		
	if(vid >= 0) {
		switch(apb->int_in[0]) {
			case ARROW: /*0x000*/
				Vdi_vsc_form(vid,&m_arrow);
				break;
			case TEXT_CRSR: /*0x001*/
				Vdi_vsc_form(vid,&m_text_crsr);
				break;
			case BUSY_BEE: /*0x002*/
				Vdi_vsc_form(vid,&m_busy_bee);
				break;
			case FLAT_HAND: /*0x004*/
				Vdi_vsc_form(vid,&m_flat_hand);
				break;
			
			case	USER_DEF	:
				Vdi_vsc_form(vid,(MFORM *)apb->addr_in[0]);
				apb->int_out[0] = 1;
				break;
			case	M_OFF	:
				Vdi_v_hide_c(vid);
				apb->int_out[0] = 1;
				break;
			case	M_ON	:
				Vdi_v_show_c(vid,0);
				apb->int_out[0] = 1;
				break;
			
			default:
				fprintf(stderr,"%s: Line %d: Graf_mouse:\r\n"
								"Unknown mode %d\r\n",__FILE__,__LINE__,apb->int_in[0]);
		};
	}
	else {
		apb->int_out[0] = 0;
	};
}

/*graf_mkstate 0x004f*/
void	Graf_mkstate(AES_PB *apb) {
	apb->int_out[0] = 1;
	apb->int_out[1] = globals.mouse_x;
	apb->int_out[2] = globals.mouse_y;
	apb->int_out[3] = globals.mouse_button;
	apb->int_out[4] = (WORD)Kbshift(-1) & 0x1f;
}
