/****************************************************************************

 Module
  menu.c
  
 Description
  Menu routines used in oAESis.
  
 Author(s)
 	cg     (Christer Gustavsson <d2cg@dtek.chalmers.se>)
 	marbud (Martin Budsj <marbud@tripnet.se>) 

 Revision history
 
  951226 cg
   Concatenated menu_lib.c and menu_srv.c into menu.c.
   Added standard header.

  960101 cg
   menu_ienable() implemented in Menu_ienable().
   menu_icheck() implemented in Menu_icheck().
   menu_tnormal() implemented in Menu_tnormal().
 
 Copyright notice
  The copyright to the program code herein belongs to the authors. It may
  be freely duplicated and distributed without fee, but not charged for.
 
 ****************************************************************************/

/****************************************************************************
 * Used interfaces                                                          *
 ****************************************************************************/

#include <aesbind.h>
#include <basepage.h>
#include <mintbind.h>
#include <process.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include "appl.h"
#include "debug.h"
#include "evnt.h"
#include "lxgemdos.h"
#include "memdefs.h"
#include "menu.h"
#include "mesagdef.h"
#include "misc.h"
#include "objc.h"
#include "rdwrs.h"
#include "resource.h"
#include "types.h"
#include "wind.h"

#include <sysvars.h>

/****************************************************************************
 * Macros                                                                   *
 ****************************************************************************/

#define menu_error(string) DEBUG(string) /*( fun_menu_error(#string,(long)(__LINE__), __FILE__) ,0 )*/

#define MENU_GEN_ERR	0
#define MENU_GEN_OK		1


/****************************************************************************
 * Typedefs of module global interest                                       *
 ****************************************************************************/

/*MENUREG is used to keep track of the application menu*/
typedef struct menureg {
	WORD apid;            /*Owner of entry       */
	char *name;           /*Application name     */
	
	struct menureg *next; /*Pointer to next entry*/
}MENUREG;

/****************************************************************************
 * Module global variables                                                  *
 ****************************************************************************/

/*
 *	Global struct for menu lib.. Created by menu_handl_init()
 */
struct {
	WORD menu_cur_apid;   /* Application id of the current menu owner.    */
	WORD menu_handl_apid; /* Set to the apid of the menu_handler process. */
	WORD winbar;          /* Window handle of the menu bar window         */
	WORD windrop;         /* Window handle of the drop down window        */
} mglob = {
	-1,
	-1,
	-1,
	-1
};

MENUREG *applmenu = NULL;

char progpath[500],progfile[70];

/****************************************************************************
 * Local functions (use static!)                                            *
 ****************************************************************************/

static WORD	get_matching_menu(OBJECT *t,WORD n) {
	WORD parent,start,i = 0;
	
	/*first we need to know which in order our title is*/
	
	parent = t[t[0].ob_head].ob_head;
	start = t[parent].ob_head;
	
	while(start != n) {
		/* we have failed to find the object! */
		
		if(start == parent)
			return -1;

		start = t[start].ob_next;
		i++;
	};
	
	/* now we shall find the i:th menubox! */
	
	parent = t[t[0].ob_head].ob_next;
	
	start = t[parent].ob_head;
	
	while(i) {
		start = t[start].ob_next;

		if(start == parent)
			return -1;
			
		i--;
	};
	
	return start;
}

static void	getmenuxpos(WORD *x,WORD *width) {
	OBJECT	*t;
	
	WORD start;
	
	*width = 0;

	Rdwrs_operation(ASTARTREAD);
	
	Rdwrs_operation(MSTARTREAD);
		
	if((t = Appl_get_menu(TOP_MENU_OWNER))) {
		start = t[t[0].ob_head].ob_head;
	
		*x = t[0].ob_x + t[t[0].ob_head].ob_x + t[start].ob_x;
		*width = t[t[start].ob_tail].ob_x + t[t[start].ob_tail].ob_width;
	}
	else {
		fprintf(stderr,"%s: Line %d: getmenuxpos:\r\n"
									"Couldn't find top menu.\r\n");
	};
	
	Rdwrs_quick(MENDREAD);
	Rdwrs_quick(AENDREAD);
}



/* 
	create_window creates a menu window with no attributes.
	
	Inputs:	maxsize = the maximum size of the window
	Return:	handle of the window
*/
static WORD	create_menu_window(RECT *maxsize) {
	WORD	wid;
	
	Rdwrs_operation(WSTARTWRITE);
	
	wid = create_window(0,0,maxsize,WIN_MENU);

	Rdwrs_operation(WENDWRITE);
		
	return wid;
}

static WORD menu_init_windows(WORD height) {
	RECT	r,r2;
	
	wind_get(0,WF_FULLXYWH, &r.x, &r.y, &r.width, &r.height);
	
	/*	Create the drop down menu window */
	
	r2.x = r.x;
	r2.y = r.y + height;
	r2.width = r.width;
	r2.height = r.height - height;
	
	mglob.windrop = create_menu_window(&r2);

	if (mglob.windrop < 0)	
	{
		menu_error("Could not create drop down menu window.");
		return MENU_GEN_ERR;
	};

	r2.y = r.y;
	r2.height = height;

	mglob.winbar = create_menu_window(&r2);

	if (mglob.winbar < 0) {
		menu_error("Could not create menubar window.");
		return MENU_GEN_ERR;
	}

      

	wind_open(mglob.winbar, r2.x, r2.y, r2.width, r2.height);

	return 0;
}

static void menu_shutdown( void ) {
	wind_close(mglob.windrop);
	wind_close(mglob.winbar);
	wind_delete(mglob.windrop);
	wind_delete(mglob.winbar);
}


static void handle_redraw(REDRAWSTRUCT *m,WORD menubox) {
	WORD   x,y,w,h;
	OBJECT *menu;
	wind_update(BEG_UPDATE);

	Rdwrs_operation(MSTARTREAD);
	
	Rdwrs_operation(ASTARTREAD);
	
	menu = Appl_get_menu(TOP_MENU_OWNER);
	
	Rdwrs_quick(AENDREAD);

	if(menu) {
		if((mglob.winbar == m->wid)
			|| ((mglob.windrop == m->wid) && (menubox > 0))) {
			WORD xn,yn,wn,hn,hp;
			WORD yp = pmenutad[0].ob_y;
			WORD deskbox = menu[menu[0].ob_tail].ob_head;
			
			wind_get(m->wid,WF_FIRSTXYWH,&x,&y,&w,&h);
		
			while((w > 0) && (h > 0)) {
				if(deskbox == menubox) {
					xn = max(x,m->area.x);
					wn = min(x + w ,m->area.x + m->area.width) - xn;
					yn = max(y,m->area.y);
					hn = min(y + h, m->area.y + m->area.height) - yn;

					yp = max(yn,yp);

					hp = hn - yp + yn;
					hn = min(hn,yp - yn);
						
					if((wn > 0) && (hn > 0)) {
						objc_draw(menu,0,9,xn,yn,wn,hn);
					};
					
					if((wn > 0) && (hp > 0)) {
						objc_draw(pmenutad,0,9,xn,yp,wn,hp);
					};
				}
				else {
					xn = max(x,m->area.x);
					wn = min(x + w ,m->area.x + m->area.width) - xn;
					yn = max(y,m->area.y);
					hn = min(y + h, m->area.y + m->area.height) - yn;
	
	
					if((wn > 0) && (hn > 0)) {
						objc_draw(menu,0,9,xn,yn,wn,hn);
					};
				};
	
				wind_get(m->wid,WF_NEXTXYWH,&x,&y,&w,&h);
			};				
		};
	}
	else {
		fprintf(stderr,"%s: Line %d: menu_handl_win_redraw:\r\n",
									"Couldn't find top menu\r\n");
	};
	
	Rdwrs_quick(MENDREAD);

	wind_update(END_UPDATE);
}

static WORD	handle_drop_down(OBJECT *menu,WORD menubox,WORD title,
															WORD x,WORD y) {
	EVENTIN  ei;
	EVENTOUT eo;
	
	MENUMSG  msg;
	
	WORD     entry;
	WORD     deskbox = menu[menu[0].ob_tail].ob_head;

	OBJECT   *nmenu;
	
	if((deskbox == menubox) && (y >= pmenutad[0].ob_y)) {
		nmenu = pmenutad;
		entry = objc_find(nmenu,0,9,x,y);
	}
	else {
		nmenu = menu;
		entry = objc_find(nmenu, menubox, 9, x, y);
	};

	if(entry >= 0) {
		REDRAWSTRUCT	r;

		if(!(nmenu[entry].ob_state & DISABLED) && (entry != 0)) {
			/* select the entry and update it */

			nmenu[entry].ob_state |= SELECTED;

			r.type = WM_REDRAW;
			r.sid = 0;
			r.length = 0;
			r.wid = mglob.windrop;

			Objc_area_needed(nmenu,entry,&r.area);
		
			appl_write(0,16,&r);
			
			ei.events = MU_MESAG | MU_M1 | MU_BUTTON;

			ei.bclicks = 1;
			ei.bmask = LEFT_BUTTON;
			ei.bstate = LEFT_BUTTON;
		}
		else
			ei.events = MU_MESAG | MU_M1;
		
			ei.m1flag = MO_LEAVE;
		
			objc_offset(nmenu, entry, &ei.m1r.x, &ei.m1r.y);
			ei.m1r.width = nmenu[entry].ob_width;
			ei.m1r.height = nmenu[entry].ob_height;
		
			/* Start to wait for messages and rect 1 */
			while(TRUE) {
				eo.events = evnt_multi(ei.events, ei.bclicks, ei.bmask,
									ei.bstate, ei.m1flag,
									ei.m1r.x, ei.m1r.y, ei.m1r.width, ei.m1r.height,
									ei.m2flag,
									ei.m2r.x, ei.m2r.y, ei.m2r.width, ei.m2r.height,
									(WORD *)&msg, ei.locount + (((long)ei.hicount) << 16),
									&eo.mx, &eo.my,
									&eo.mb, &eo.ks,
									&eo.kc, &eo.mc);
	
			if(MU_MESAG & eo.events) {
				switch(msg.type) {
				/* window dirty.. 							*/
				case WM_REDRAW:
					handle_redraw((REDRAWSTRUCT *)&msg,menubox);
					break;
						
  		   default:
      		fprintf(stderr,"Unknown message %d in handle_drop_down\r\n",msg.type);
				};				
			};
						
			if(MU_M1 & eo.events) {
				if(!(nmenu[entry].ob_state & DISABLED)) {
					nmenu[entry].ob_state &= ~SELECTED;
		
					r.type = WM_REDRAW;
					r.sid = 0;
					r.length = 0;
					r.wid = mglob.windrop;
		
					Objc_area_needed(nmenu,entry,&r.area);
				
					appl_write(0, 16, &r);
				};
				
				return 0;
			};
			
			if(MU_BUTTON & eo.events) {
				WORD    dummy;
	
				evnt_button(0,LEFT_BUTTON, 0, &dummy, &dummy, &dummy, &dummy);				
				
				nmenu[entry].ob_state &= ~SELECTED;

				if(nmenu == pmenutad) {
					MENUREG *mr = applmenu;
					WORD    walk = entry - PMENU_FIRST;
				
					while(walk && mr) {
						mr = mr->next;
						walk--;
					};
				
					if(mr) {
						COMMSG m;
						
						m.type = MH_TOP;
						m.sid = 0;
						m.length = 0;
						m.msg1 = mr->apid;

						Appl_do_write(0, 16, &m);
					}
					else {
						fprintf(stderr,"%s: Line %d: handle_drop_down:\r\n"
							"Couldn't find application to top!\r\n",__FILE__,__LINE__);
					};
				}
				else {
					MENUMSG m;

					m.type = MN_SELECTED;
					m.sid = 0;
					m.length = 0;
					m.title = title;
					m.item = entry;
					m.tree = nmenu;
					m.parent = menubox;

					Appl_do_write(TOP_MENU_OWNER, 16, &m);
				};
				
				return 1;
			};
		};	
	};
	
	return 0;
}

static WORD	handle_selected_title(WORD mx,WORD my) {
  MENUMSG	msg;

	WORD    mmx,mmy;  
  WORD    events;
  WORD	  box;
  WORD	  title;
	WORD    m1flag;
	RECT    m1r;
	WORD    dummy;
	
	OBJECT  *menu;

	Rdwrs_operation(MSTARTREAD);
	
	Rdwrs_operation(ASTARTREAD);
  
  menu = Appl_get_menu(TOP_MENU_OWNER);
  
  Rdwrs_quick(AENDREAD);
  
  if(menu) {
  	WORD deskbox;
  	
	  title = objc_find(menu, 0, 9, mx, my);
	  box = get_matching_menu(menu, title);
	  deskbox = menu[menu[0].ob_tail].ob_head;
	  
	  if(box >= 0) {
	    REDRAWSTRUCT r;
			RECT         area;
	    
	    /* select the title and update it */
	    
	    menu[title].ob_state |= SELECTED;
	
	    r.type = WM_REDRAW;
	    r.sid = 0;
	    r.length = 0;
	    r.wid = mglob.winbar;
	    
	    Objc_area_needed(menu,title,&r.area);
	    
	    appl_write(0,16,&r);
	
			Objc_area_needed(menu,box,&area);

			if(box == deskbox) {
				Menu_update_appl();
				
		    objc_offset(menu,box,&pmenutad[0].ob_x,&pmenutad[0].ob_y);
	    
		    pmenutad[0].ob_y += (menu[menu[box].ob_head].ob_height << 1);
				pmenutad[0].ob_width = menu[box].ob_width;
			
				area.height = pmenutad[0].ob_height + (pmenutad[1].ob_height << 1) + 2;
			};
					
	    wind_open(mglob.windrop,area.x,area.y,area.width,area.height);
	
			menu[box].ob_flags &= ~HIDETREE;
	
	    
	    m1flag = MO_LEAVE;
	    
	    objc_offset(menu, title, &m1r.x, &m1r.y);
	    m1r.width = menu[title].ob_width;
	    m1r.height = menu[title].ob_height;
	    
	    /* Start to wait for messages and rect 1 */
	    while(TRUE) {
	      events = evnt_multi(MU_MESAG | MU_M1,
	      		0, 0, 0,
	      		m1flag, m1r.x, m1r.y, m1r.width, m1r.height,
				    0, 0, 0, 0, 0,
				    (WORD *)&msg,
				    0,
			    	&mmx, &mmy, &dummy, &dummy, &dummy, &dummy);
	      
	      if(MU_MESAG & events) {
					switch(msg.type) {
				  /* window dirty.. 			*/
					case WM_REDRAW:
		  
					  handle_redraw((REDRAWSTRUCT *)&msg, box);
					  break;
		  
	      	default:
					  fprintf(stderr,"Unknown message %d in handle_selected_message\r\n",msg.type);
					};
	      };
	      
	      if(MU_M1 & events) {
	      	WORD closebox = 0;
	      	
	      	if((deskbox == box) && (mmy >= pmenutad[0].ob_y)) {
	      		if(!Misc_inside((RECT *)&pmenutad[0].ob_x,mmx,mmy) ||
	      			(objc_find(pmenutad,0,9,mmx,mmy) < 0) ||
	      			(handle_drop_down(menu,box,title,mmx,mmy))) {
	      			closebox = 1;
	      		};
	      	}
	      	else if((objc_find(menu, box, 9, mmx, mmy)
					    < 0) || (handle_drop_down(menu,box,title,mmx,mmy))) {
						closebox = 1;
					};
					    
					if(closebox) {
					  menu[title].ob_state &= ~SELECTED;
					  
					  r.type = WM_REDRAW;
					  r.sid = 0;
					  r.length = 0;
					  r.wid = mglob.winbar;
					  
					  Objc_area_needed(menu,title,&r.area);
					  
					  appl_write(0,16,&r);
					  
					  wind_close(mglob.windrop);
						menu[box].ob_flags |= HIDETREE;
					  
						Rdwrs_quick(MENDREAD);
					  return 0;
					};
	      };
	    };	
	  };
	}
	else {
		fprintf(stderr,"%s: Line %d: handle_selected_title\r\n"
									"Couldn't find top menu\r\n.",__FILE__,__LINE__);
	};

	Rdwrs_quick(MENDREAD);
  
  return 0;
}

/*
 *	Menu install code.. 
 *
 *	Creates and updates the menu list entries for the specified menu. 
 *	Menu capid is unique. New capid => new menu. Old capid => update menu.
 *
 */
static WORD       /* .w return code.*/
menu_bar_install( /* menu_bar(MENU_INSTALL) */
OBJECT *tree,     /* object tree. Pointer to the menu's OBJECT	*/
WORD   capid)     /* Current application id. Menu owner	*/
{
	WORD         i;
	REDRAWSTRUCT r;

  /* Lock the menu list from other processes.. 	*/

	Rdwrs_operation(MSTARTWRITE);
	
	Appl_set_menu(capid,tree);

	/*Unlock the menu list..				 	*/
	Rdwrs_quick(MENDWRITE);

	/* Mark all drop down menus with HIDETREE */
	i = tree[tree[tree[0].ob_head].ob_next].ob_head;

	while(i != -1) {		
		tree[i].ob_flags |= HIDETREE;
		
		if(i == tree[tree[i].ob_next].ob_tail) {
			break;
		};
		
		i = tree[i].ob_next;
	};

	r.type = WM_REDRAW;
	r.sid = 0;
	r.length = 0;
	r.wid = mglob.winbar;

	r.area = globals.screen;
		
	Appl_do_write(mglob.menu_handl_apid,16,&r);

	return MENU_GEN_OK;
}

/*
 * 	This one is simple.. Return the apid of the currently topped menu owner.
 * 
 */
static WORD         /* .w return code	*/
menu_bar_inquire(   /*	*/
void)               /*	*/
{
	WORD	ret;
	
	Rdwrs_operation(MSTARTREAD);
	
	ret = mglob.menu_cur_apid;
	
	Rdwrs_operation(MENDREAD);
	
	return ret;
}


/****************************************************************************
 * Public functions                                                         *
 ****************************************************************************/

/****************************************************************************
 * Menu_init_module                                                         *
 *  Initialize menu module.                                                 *
 ****************************************************************************/
WORD                   /*                                                   */
Menu_init_module(void) /*                                                   */
/****************************************************************************/
{
	OBJECT	*ourtree;
	
	WORD	ourid;
	
	mglob.menu_handl_apid = appl_init();
	ourid = mglob.menu_handl_apid;
	
	menu_register(ourid,"  oAESis");
	
	recalc_tree(menutad,globals.cwidth,globals.cheight);
	recalc_tree(informtad,globals.cwidth,globals.cheight);
	recalc_tree(pmenutad,globals.cwidth,globals.cheight);

	ourtree = menutad;
		
	menu_bar_install(ourtree,ourid);

	menu_init_windows(menutad[menutad[0].ob_head].ob_height + 1);
/*
	strcpy(progpath,"u:\\d\\nytto\\desktop\\gemini2\\*.app");
*/
/*
	strcpy(progpath,"u:\\d\\nytto\\develop\\pure_c\\*.prg");
*/
/*
	strcpy(progpath,"u:\\e\\incoming\\hacke\\tw14w12.bin\\*.prg");
*/
/*
	strcpy(progpath,"u:\\d\\nytto\\dummies\\*");
*/
/*
	strcpy(progpath,"u:\\usr\\bin\\*");
*/
/*
								strcpy(path,"u:\\e\\develop\\oaesis\\src\\test\\*.prg");
*/
/*
								strcpy(path,"u:\\e\\develop\\qlevered\\*.app");
*/

	strcpy(progpath,"u:\\*");
	
	strcpy(progfile,"");

	wind_set(DESKTOP_HANDLE,WF_NEWDESK,deskbgtad);
							
	return 0;
}

/****************************************************************************
 *  Menu_exit_module                                                        *
 *   Shutdown menu module.                                                  *
 ****************************************************************************/
void                   /*                                                   */
Menu_exit_module(void) /*                                                   */
/****************************************************************************/
{
  menu_shutdown();
  appl_exit();
}


/****************************************************************************
 *  Menu_handler                                                            *
 *   Handle main menu events.                                               *
 ****************************************************************************/
WORD                   /*                                                   */
Menu_handler(void)     /*                                                   */
/****************************************************************************/
{
	WORD    kc;
  WORD    fx,fy,fw,fh;
	WORD    events;
  WORD    mx,my;
  MENUMSG	msg;
	WORD    m1flag = MO_ENTER;
	RECT    m1r;
	WORD    dummy;

  wind_get(mglob.winbar, WF_WORKXYWH, &m1r.x, &m1r.y, &m1r.width,
	   &m1r.height);
  
  /* Start waiting for messages and rect 1 */
  while(TRUE) {
	  getmenuxpos(&m1r.x,&m1r.width);

    events = evnt_multi(MU_MESAG | MU_M1 | MU_KEYBD,
    					0, 0, 0,
    					m1flag, m1r.x, m1r.y, m1r.width, m1r.height,
			   			0, 0, 0, 0, 0,
			   			(WORD *)&msg,
			   			0,
					  	&mx, &my, &dummy, &dummy, &kc, &dummy);

    if(MU_MESAG & events) {
      switch(msg.type) {
      case MN_SELECTED:
				switch(msg.title) {
				case MENU_FILE:
					switch(msg.item) {
						case MENU_QUIT:
							return 0;

						case MENU_STARTPROG:
							{
								char	execpath[128];
								char	oldpath[128];
								char *tmp;
								int		button;


								fsel_exinput(progpath,progfile,&button,"Select program to start");
						
								if(button == FSEL_OK) {
									LONG err;
									char newpath[128];
									
									strcpy(newpath,progpath);
									
									tmp = strrchr(newpath,'\\');
									
									if(tmp) {
										*tmp = '\0';
										sprintf(execpath,"%s\\%s",newpath,progfile);
									}
									else {
										strcpy(execpath,progfile);
									};
									
									Dgetpath(oldpath,0);
								
									Misc_setpath(newpath);
									
									err = Pexec(100,execpath,0L,0L);
									
									Misc_setpath(oldpath);
									
									if(err < 0) {
										form_error((WORD) -err - 31);
									};
								};
							};
							break;
					};
			    break;
			  
			  case MENU_OAESIS:
					switch(msg.item) {
					case MENU_INFO:
						form_center(informtad,&fx,&fy,&fw,&fh);
						form_dial(FMD_START,0,0,0,0,fx,fy,fw,fh);
						objc_draw(informtad,0,9,fx,fy,fw,fh);
						form_do(informtad,0);
						informtad[INFOOK].ob_state &= ~SELECTED;
						form_dial(FMD_FINISH,0,0,0,0,fx,fy,fw,fh);
						break;
					};
					break;
				};
				break;
	
      /* window dirty.. 	*/
      case WM_REDRAW:
        handle_redraw((REDRAWSTRUCT *)&msg,0);
        break;

			case MH_TOP:
				{
					REDRAWSTRUCT rs;
					
					Appl_top(((COMMSG *)&msg)->msg1);
					
					rs.type = WM_REDRAW;
					rs.sid = 0;
					rs.length = 0;
					rs.wid = mglob.winbar;
					rs.area = globals.screen;
					
					handle_redraw(&rs,0);
				};
				break;
					
      default:
      	fprintf(stderr,"Unknown message %d in Menu_handler\r\n",msg.type);
      };
    };
    
    if(MU_KEYBD & events) {
      if((kc & 0xff) == 'q')
				return 0;
    };
    
    if(MU_M1 & events) {
      handle_selected_title(mx,my);
    };
  };
}

/****************************************************************************
 * Menu_bar_remove                                                          *
 *  Remove menu.                                                            *
 ****************************************************************************/
WORD              /*                                                        */
Menu_bar_remove(  /*                                                        */
WORD apid)        /* Application whose menu is to be removed.               */
/****************************************************************************/
{
	REDRAWSTRUCT	r;
		
	/*	Lock the menu list.. */
	Rdwrs_operation(MSTARTWRITE);

	Appl_set_menu(apid,NULL);	

	Rdwrs_quick(MENDWRITE);

	r.type = WM_REDRAW;
	r.sid = 0;
	r.length = 0;
	r.wid = mglob.winbar;

	r.area = globals.screen;
		
	Appl_do_write(mglob.menu_handl_apid,16,&r);

	return MENU_GEN_OK;
}


/****************************************************************************
 *  Menu_bar                                                                *
 *   0x001e menu_bar() library code.                                        *
 ****************************************************************************/
void              /*                                                        */
Menu_bar(         /*                                                        */
AES_PB *apb)      /* Pointer to AES parameter block.                        */
/****************************************************************************/
{
	switch(apb->int_in[0]) {
		case	MENU_INSTALL: 
			apb->int_out[0] = menu_bar_install((OBJECT *)apb->addr_in[0]
						,apb->global[2]);
			break;
			
		case	MENU_REMOVE:
			apb->int_out[0] = Menu_bar_remove(apb->global[2]);
			break;
			
		case	MENU_INQUIRE:
			apb->int_out[0] = menu_bar_inquire();
			break;
			
		default:
			apb->int_out[0] = MENU_GEN_ERR;
	};
}

/****************************************************************************
 *  Menu_icheck                                                             *
 *   0x001f menu_icheck().                                                  *
 ****************************************************************************/
void              /*                                                        */
Menu_icheck(      /*                                                        */
AES_PB *apb)      /* Pointer to AES parameter block.                        */
/****************************************************************************/
{
	switch(apb->int_in[1]) {
	case UNCHECK:
		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state &= ~CHECKED;
		apb->int_out[0] = 1;
		break;
	case CHECK:
		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state |= CHECKED;
		apb->int_out[0] = 1;
		break;
	default:
		fprintf(stderr,"%s: Line %d: Menu_icheck:\r\n"
						"Unknown mode %d!\r\n",__FILE__,__LINE__,apb->int_in[1]);
		apb->int_out[0] = 0;
	}
}

/****************************************************************************
 *  Menu_ienable                                                            *
 *   0x0020 menu_ienable().                                                 *
 ****************************************************************************/
void              /*                                                        */
Menu_ienable(     /*                                                        */
AES_PB *apb)      /* Pointer to AES parameter block.                        */
/****************************************************************************/
{
	switch(apb->int_in[1]) {
	case ENABLE:
		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state &= ~DISABLED;
		apb->int_out[0] = 1;
		break;
	case DISABLE:
		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state |= DISABLED;
		apb->int_out[0] = 1;
		break;
	default:
		fprintf(stderr,"%s: Line %d: Menu_ienable:\r\n"
						"Unknown mode %d!\r\n",__FILE__,__LINE__,apb->int_in[1]);
		apb->int_out[0] = 0;
	};
}

/****************************************************************************
 *  Menu_tnormal                                                            *
 *   0x0021 menu_tnormal().                                                 *
 ****************************************************************************/
void              /*                                                        */
Menu_tnormal(     /*                                                        */
AES_PB *apb)      /* Pointer to AES parameter block.                        */
/****************************************************************************/
{
	switch(apb->int_in[1]) {
	case UNHIGHLIGHT:
		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state &= ~SELECTED;
		apb->int_out[0] = 1;
		break;
	case HIGHLIGHT:
		((OBJECT *)apb->addr_in[0])[apb->int_in[0]].ob_state |= SELECTED;
		apb->int_out[0] = 1;
		break;
	default:
		fprintf(stderr,"%s: Line %d: Menu_ienable:\r\n"
						"Unknown mode %d!\r\n",__FILE__,__LINE__,apb->int_in[1]);
		apb->int_out[0] = 0;
	};
}


/****************************************************************************
 *  Menu_text                                                               *
 *   0x0022 menu_text().                                                    *
 ****************************************************************************/
void              /*                                                        */
Menu_text(        /*                                                        */
AES_PB *apb)      /* Pointer to AES parameter block.                        */
/****************************************************************************/
{
	strcpy(((TEDINFO *)((OBJECT *)apb->addr_in[0])->ob_spec)->te_ptext,
					(char *)apb->addr_in[1]);
					
	apb->int_out[0] = 1;
}

/****************************************************************************
 * Menu_update_appl                                                         *
 *  Update the application menu.                                            *
 ****************************************************************************/
void                   /*                                                   */
Menu_update_appl(void) /*                                                   */
/****************************************************************************/
{
	MENUREG *mwalk;
	WORD    rwalk;
	WORD    topappl;
	
	Rdwrs_operation(MSTARTREAD);

	topappl = Appl_get_top();

	mwalk = applmenu;
	rwalk = PMENU_FIRST;
	
	while(mwalk) {
		((char **)pmenutad[rwalk].ob_spec) = mwalk->name;

		if(mwalk->apid == topappl) {
			pmenutad[rwalk].ob_state |= CHECKED;
		}
		else {
			pmenutad[rwalk].ob_state &= ~CHECKED;
		};
	
		mwalk = mwalk->next;
		rwalk++;
	};
	
	pmenutad[0].ob_height = pmenutad[rwalk].ob_y;
	
	Rdwrs_quick(MENDREAD);
}

/****************************************************************************
 * Menu_unregister                                                          *
 *  Remove menu entry of application.                                       *
 ****************************************************************************/
void              /*                                                        */
Menu_unregister(  /*                                                        */
WORD apid)        /* Application id.                                        */
/****************************************************************************/
{
	MENUREG **mwalk;
	
	Rdwrs_operation(MSTARTWRITE);
	
	mwalk = &applmenu;
	
	while(*mwalk) {
		if((*mwalk)->apid == apid) {
			break;
		};
	
		mwalk = &(*mwalk)->next;
	};
	
	if(*mwalk) {
		MENUREG *tmp = *mwalk;
		
		*mwalk = (*mwalk)->next;
		
		Mfree(tmp->name);
		Mfree(tmp);
	};

	Rdwrs_quick(MENDWRITE);

/*	Menu_update_appl(); */
}

/****************************************************************************
 * Menu_do_register                                                         *
 *  Implementation of menu_register().                                      *
 ****************************************************************************/
WORD              /* Menu identification, or -1.                            */
Menu_do_register( /*                                                        */
WORD real_apid,   /* Real application id.                                   */
WORD apid,        /* Application id, or -1.                                 */
char *title)      /* Title to register application under.                   */
/****************************************************************************/
{
	MENUREG **mwalk;
	
	Rdwrs_operation(MSTARTWRITE);
	
	mwalk = &applmenu;
	
	while(*mwalk) {
		if((*mwalk)->apid == apid) {
			break;
		};
	
		mwalk = &(*mwalk)->next;
	};
	
	if(!*mwalk) {
		*mwalk = Mxalloc(sizeof(MENUREG),GLOBALMEM);
		(*mwalk)->name = Mxalloc(21,GLOBALMEM);
		(*mwalk)->apid = apid;
		(*mwalk)->next = 0L;
	};

	strncpy((*mwalk)->name,title,20);
	
	Rdwrs_quick(MENDWRITE);

/*	Menu_update_appl();*/

	return -1;
}

/****************************************************************************
 * Menu_register                                                            *
 *  0x0023 menu_register().                                                 *
 ****************************************************************************/
void              /*                                                        */
Menu_register(    /*                                                        */
AES_PB *apb)      /* Pointer to AES parameter block.                        */
/****************************************************************************/
{
	apb->int_out[0] = Menu_do_register(apb->global[2],apb->int_in[0],
													(char *)apb->addr_in[0]);
}
